"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitCSS = exports.createValueHash = void 0;
const constants_1 = require("./constants");
const hash_1 = require("./hash");
function toCSSProp(prop) {
    return prop.replace(constants_1.KEBAB_CASE_REGEXP, "$1-$2").toLowerCase();
}
function makeCSSCompat(str) {
    if (constants_1.PREFIX_WITH_UNDERSCORE.indexOf(str[0]) > -1)
        return `_${str}`;
    return str;
}
const sanitizeRegExp = /([^\w]|_)/g;
function createValueHash(key, unparsed, hashable, { mediaQueryMap, pseudoSelectorMap, cssPropertyMap }) {
    key = key.trim();
    const value = String(unparsed).trim();
    const isMediaQuery = hashable && hashable.media;
    const PSEUDO_SELECTOR = hashable && hashable.pseudo;
    const isSpec = isMediaQuery || PSEUDO_SELECTOR;
    // example: const rawCSSRule  = "margin:auto;"
    let prefix = "";
    const rawCSSRule = `${toCSSProp(key)}:${value};`;
    if (process.env.NODE_ENV !== "production")
        prefix = `${rawCSSRule.replace(sanitizeRegExp, "_")}____`;
    // a unique rule will be one with a different media/pseudo rule + key&value
    const identity = ((hashable && (isMediaQuery || PSEUDO_SELECTOR)) || "").trim() + rawCSSRule;
    let cache;
    const $map = isSpec && isMediaQuery ? mediaQueryMap : pseudoSelectorMap;
    let fetchMap;
    if ($map) {
        fetchMap = $map.get(isSpec);
        if (!fetchMap) {
            fetchMap = new Map();
            $map.set(isSpec, fetchMap);
        }
    }
    if (isSpec) {
        cache = fetchMap.get(identity);
    }
    else {
        cache = cssPropertyMap.get(identity);
    }
    if (cache)
        return cache.class;
    const hash = prefix + makeCSSCompat((0, hash_1.murmur2)(identity));
    const obj = { class: hash, cssRule: rawCSSRule };
    if (isSpec) {
        fetchMap.set(identity, obj);
    }
    else {
        cssPropertyMap.set(identity, obj);
    }
    return hash;
}
exports.createValueHash = createValueHash;
const toCSS = (m) => Array.from(m.values())
    .map((v) => `.${v.class} { ${v.cssRule} }`)
    .sort()
    .join("\n");
function mergeDuplicateRules(value, dedupedPropMap, suffix = "") {
    const cls = `.${value.class}${suffix}`;
    const rule = value.cssRule;
    let arr = dedupedPropMap.get(rule);
    if (!arr) {
        arr = new Set();
        dedupedPropMap.set(rule, arr);
    }
    arr.add(cls);
}
function emitCSS({ mediaQueryMap, pseudoSelectorMap, cssPropertyMap, }) {
    const dedupedPropMap = new Map();
    Array.from(cssPropertyMap.values()).forEach((v) => mergeDuplicateRules(v, dedupedPropMap));
    Array.from(pseudoSelectorMap.entries()).forEach(([k, v]) => Array.from(v.values())
        .sort()
        .forEach((prop) => mergeDuplicateRules(prop, dedupedPropMap, k)));
    const mediaQueries = Array.from(mediaQueryMap.entries())
        .map(([k, v]) => `@media ${k} {\n${toCSS(v)}\n}\n`)
        .sort()
        .join("\n");
    const cssProps = Array.from(dedupedPropMap.entries())
        .map(([rule, classNames]) => `${Array.from(classNames).join(",\n")}{ ${rule} }`)
        .sort()
        .join("\n");
    return [cssProps, mediaQueries].join("\n");
}
exports.emitCSS = emitCSS;
